%This is a Matlab script powered by Panayiotis Dimitriadis (https://scholar.google.gr/citations?user=L6hWcz0AAAAJ&hl=en)
%based on the analysis of (https://www.itia.ntua.gr/el/docinfo/1656/)
%P. Dimitriadis, and D. Koutsoyiannis, Stochastic synthesis approximating any process dependence and distribution, Stochastic Environmental Research & Risk Assessment, 32 (6), 14931515, doi:10.1007/s00477-018-1540-2, 2018.
%for the stochastic simulation of a series with long-range dependence (HK model) 
%by preserving explicitly (i.e. in a direct way without including transformations)
%the first four central moments of the target marginal distribution or of the sample series
%As Input it requires the: mean (Sm), variance (Sv), skewness and kurtosis coefficients (Ss and Sk),
%Hurst parameter of the HK model (H), length of synthetic series (N), and Output the syntehtic series (S)
%An example [S]=SMA4_HK(3.1,6.5^2,4.3,30.2,0.6,10^5); is the application 4.2 of the above paper to daily precipitation.
function [S]=SMA4_HK(Sm,Sv,Ss,Sk,H,N)

l=Sv; NA=N; MA=(1:NA)';
Ac=sqrt(2*gamma(2*H+1)*sin(pi*H)*l/(((gamma(H+1.5))^2)*(1+sin(pi*H))))*(((abs([0;MA]+1)).^(H+0.5)+(abs([0;MA]-1)).^(H+0.5))/2-(abs([0;MA])).^(H+0.5)); %Averaged Process (Kouts, 2015)
SA=[Ac(1+N:-1:2)' Ac(1) Ac(2:N+1)']; Vm=0; Vv=1;
Vs=Ss*((Ac(1)^2+2*sum(Ac(2:NA+1).^2)).^(3/2))/(Ac(1)^3+2*sum(Ac(2:NA+1).^3));

if Sk~=3;
SUM=0; for I=2:NA; SUM=SUM+(Ac(I)^2)*sum(Ac(I+1:NA+1).^2); end
Vk=abs(((((Ac(1)^2+2*sum(Ac(2:NA+1).^2)).^2)*Sk)-6*sum(Ac(2:NA+1).^4)-12*(Ac(1)^2)*sum(Ac(2:NA+1).^2)-24*SUM)/(Ac(1)^4+2*sum(Ac(2:NA+1).^4)));
else
Vk=3;
end

if (Ss==0)&&(Sk==3);
CW=1; a=Vm; b=Vv; c=0; d=0;
else
if (Vk-5/3*(Vs)^2-3<=0);
CW=2;
KKF=@(x) [Vs-(2*(abs(x(2))*beta(1+1/abs(x(1)),abs(x(2))))^3-3*abs(x(2))*beta(1+1/abs(x(1)),abs(x(2)))*abs(x(2))*beta(1+2/abs(x(1)),abs(x(2)))+abs(x(2))*beta(1+3/abs(x(1)),abs(x(2))))/(abs(x(2))*beta(1+2/abs(x(1)),abs(x(2)))-(abs(x(2))*beta(1+1/abs(x(1)),abs(x(2))))^2)^1.5;
Vk-(-3*(abs(x(2))*beta(1+1/abs(x(1)),abs(x(2))))^4+6*(abs(x(2))*beta(1+2/abs(x(1)),abs(x(2))))*(abs(x(2))*beta(1+1/abs(x(1)),abs(x(2))))^2-4*abs(x(2))*beta(1+1/abs(x(1)),abs(x(2)))*abs(x(2))*beta(1+3/abs(x(1)),abs(x(2)))+abs(x(2))*beta(1+4/abs(x(1)),abs(x(2))))/(abs(x(2))*beta(1+2/abs(x(1)),abs(x(2)))-(abs(x(2))*beta(1+1/abs(x(1)),abs(x(2))))^2)^2];
opt=optimset('Display','off'); warning('off','all'); [KKF,~]=fsolve(KKF,[1 10^2],opt);
a=abs(KKF(1)); b=abs(KKF(2)); d=sqrt(Vv/(b*beta(1+2/a,b)-(b*beta(1+1/a,b))^2)); c=Vm-b*d*beta(1+1/a,b);
else
CW=3;
c=sqrt(3/Vv/(Vk-5/3*(Vs)^2-3));
b=(c^2)*Vs*sqrt(Vv)/3;
d=(c^3)*Vv/(b^2+c^2);
a=Vm-b*d/c;
end
end

if CW==1; V=norminv(rand(N+2*NA,1),a,sqrt(b)); end
if CW==2; V=d*(1-(1-rand(N+2*NA,1)).^(1/b)).^(1/a)+c; end
if CW==3;
V1=norminv(rand(N+2*NA,1),0,1); V2=V1.^2; V3=d/c+V2/2/c^2-sqrt(4*V2*d/c+(V2/c).^2)/2/c; V4=rand(N+2*NA,1);
Z=V3; Z(V4>1./(1+V3*c/d))=((d/c)^2)./V3(V4>1./(1+V3*c/d));
V=a+b*Z+sqrt(Z).*norminv(rand(N+2*NA,1),0,1);
end

S=zeros(N,1); if H~=0.5; for J=1:N; S(J,1)=SA*V(J:2*NA+J,1); end; else S=V(1:NA,1); end
S=S/sqrt(l)*sqrt(Sv)+Sm;

end